#!/usr/bin/python
# -*-coding: utf-8-*-

from numpy import linalg as la
import numpy as np
import math

# Himmelblau-Fkt.
def H(x,y):
    h = math.pow((x*x + y - 11),2) + math.pow((x + y*y -7),2)
    return h
# endfunction

# Gradient der Himmelblau-Fkt.
def gradH(x,y):
    grad = np.zeros(2)
    grad[0] = 4*(x*x + y -11)*x + 2*(x + y*y -7)
    grad[1] = 2*(x*x + y -11) + 4*(x + y*y -7)*y
    return grad
# endfunction

# Programm 7.2 zum Abstiegsverfahren (Armijo-Schrittweiten) (abstieg_gb.py)
# Min H(x,y) , H und gradH (Spaltenvektor)
#              sind als Programme (Function) bereitzustellen
# input:  Startwert (x0,y0), 
#         alpha (Daempfung des Gradienten), rho (Armijo-Parameter)
# output: Loesung (x*,y*), H(x*,y*) und grad H(x*,y*)
# Aufruf: [xs,ys,Hs,ngrad,k] = abstieg_gb(x0,y0,alpha,rho)
# z.B. ==> alpha = 0.5, rho = 0.1

def abstieg_gb(x0,y0,alpha,rho):
    t0 = 0.25; l = 0; k = 0; epsilon = 0.0001
    ngrad = 10
    x = x0; y = y0; t = t0
    grad = gradH(x,y); grad = np.dot(1.0,grad) 
    d = -np.dot(alpha,grad)
    fex = H(x,y);
    xn = x + t*d[0]; yn = y + t*d[1]
    fexn = H(xn,yn)
    while (ngrad > epsilon): 
        b0 = fex + t*rho*np.dot(grad.T,d) < fexn
        while (b0 and t > 0.01):
            l = l+1
            t = t*0.5 
            xn = x + t*d[0]; yn = y + t*d[1]
            fexn = H(xn,yn)
            b0 = fex + t*rho*np.dot(grad.T,d) < fexn
#       endwhile
        l = 0
        ngrad = la.norm(grad) # Norm des Gradienten
        fex = fexn 
        x = x + t*d[0]; y = y + t*d[1]
        grad = gradH(x,y)
        d = -np.dot(alpha,grad) 
        fexn = H(x,y)
        t = t0
        k = k+1 # Iterationszaehler
#   endwhile
    xs = x; ys = y
    Hs = fex;
    ngrad
    return (xs,ys,Hs,ngrad,k)
# endfunction
