#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math
import time
import matplotlib.pyplot as plt
from numpy.linalg import norm

# Programm 10.6 Test der starken Konvergenz Euler-Maruyama
# am Beispiel dX = mue X dt + sigma X dW, X(0) = Xzero ,
# mue = 2 , sigma = 1 and Xzer0 = 1 .
#
# Brownsche Bewegung ueber [0,1], h = 2^(-9).
# 5 EM Schritte: 16 h , 8 h , 4 h , 2 h , h .
# Auswertung bei T=1: E | X_L - X(T) | .

np.random.seed(100) # random seeding
mue = 2 ; sigma = 1 ; Xzero = 1  
T = 1 ; N = int(math.pow(2,9)) ; h = T/N
M = 1000 ; # Zahl der gesamplten Pfade
Xerr = np.zeros( (M,5) ) ; # Pre-Allokierung
dW = np.zeros(N); W = np.zeros(N); # Pre-Allokierung
start = time.time()
for s in range(0,M): # samples ueber approximierte Wiener-Prozess-Pfade
    dW = np.dot(math.sqrt(h),np.random.randn(N))
    W = np.cumsum(dW)  # approximierter Wiener-Prozess-Pfad
    Xtrue = Xzero*math.exp((mue -0.5*sigma*sigma)+sigma*W[N-1] ) ;
    for p in range(0,5):
        R = int(math.pow(2,p)); Dt = R*h ; L = int(N/R)  
        # EM Schritte Dt = R h 
        Xtemp = Xzero
        for j in range(0,L):
            Winc = sum(dW[R*j:R*(j+1)]) ;
            Xtemp = Xtemp + Dt*mue*Xtemp + sigma*Xtemp*Winc
#       endfor
        Xerr[s,p] = abs(Xtemp - Xtrue) ; # Fehler bei T=1
#   endfor
# endfor

xerrm = np.mean(Xerr,axis=0)
ende = time.time()
print('{:10.8f}s'.format(ende-start))
#
Dtvals = h*np.array([1,2,4,8,16]) # Dtvals = h*(2.^([0:4])) ;
Dtvals05 = np.zeros(5)
Dtvalsq = np.zeros(5)
rhs = np.zeros(5)
for j in range(5):
    Dtvals05[j] = math.sqrt(Dtvals[j])
# endfor

plt.loglog(Dtvals, xerrm, '*')
plt.loglog(Dtvals, Dtvals05, 'g-') # Referenzsteigung von 1/2
# axis([1e-3 1e-1 1e-2 1e+1 ])
plt.xlabel('h')
plt.ylabel('Mittel von |X(T) - X_L|')
# Ausgleichskurve:  C*Dt^q 
A = np.array([[1, math.log(Dtvals[0])],[1,math.log(Dtvals[1])], \
               [1,math.log(Dtvals[2])], \
        [1,math.log(Dtvals[3])],[1,math.log(Dtvals[4])]])
#  A = [ones(5,1),log(Dtvals)'] ; 
for i in range(5):
    rhs[i] = math.log(xerrm[i])
# endfor    

# Direct least square regression
sol = np.dot((np.dot(np.linalg.inv(np.dot(A.T,A)),A.T)),rhs)
print(sol)

q = sol[1]
C = math.exp(sol[0])
for j in range(5):
    Dtvalsq[j] = C*math.pow(Dtvals[j],q)
# endfor
# tatsaechliche Steigung
plt.loglog(Dtvals, Dtvalsq, 'r-')
# loglog(Dtvals,C*(Dtvals.^q),'r--'), hold off # tatsaechliche Steigung
resid = norm(np.dot(A,sol) - rhs)
# Programmende
