#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math
import matplotlib.pyplot as plt


# Programm 10.3: euler_maruyama_gb.m ; Euler-Maruyama-Verfahren 
# lineare SDE  dX = mue*X dt + sigma*X dW, X(0) = Xzero
# mue = 2, sigma = 1 und Xzero = 1
#
# Approximation des Wiener-Prozesses auf einem Gitter [0,1] mit dt = 2^(-8)
# beim Euler-Maruyama-Verfahren werden groessere Zeitschritte benutzt: R*dt 

np.random.seed(50) # random seeding
mue = 2; sigma = 1; Xzero = 1 # Parameter des Problems
T = 1 ; N = int(math.pow(2,10)) ; dt = 1/N

dW = np.zeros(N); W = np.zeros(N); Xtrue = np.zeros(N)  # Pre-Allokierung
dW = np.dot(math.sqrt(dt),np.random.randn(N))  # Inkremente
W = np.cumsum(dW)  # approximierter Pfad eines Wiener-Prozess

tt = np.zeros(N)
for i in range(N):
    tt[i] = (i+1)*dt
# endfor
for i in range(N):
    Xtrue[i] = Xzero*math.exp( (mue - 0.5*math.pow(sigma,2))*tt[i] + sigma*W[i] )
# Xtrue = Xzero*np.dot( math.exp((mue - 0.5*math.pow(sigma,2))*tt + np.dot(sigma,W)) ) # exakte Loesung
waw = np.zeros(N+1)
tw = np.zeros(N+1); tw[0] = 0.0; waw[0] = Xzero
for j in range(1,N+1):
    tw[j] = dt + j*dt
    waw[j] = Xtrue[j-1]
# endfor
plt.plot(tw,waw,'g-')
# plot([0:dt:T],[Xzero,Xtrue],'g-'), hold on
R = 4 ; Dt = R*dt ; L = int(N/R) # L Euler-Maruyama Schwritte mit Dt = R*dt
Xem = np.zeros(L)  # Pre-Allokierung
Xtemp = Xzero 
for j in range(L):
    Winc = sum(dW[R*j:R*(j+1)])  
    Xtemp = Xtemp + Dt*mue*Xtemp + sigma*Xtemp*Winc 
    Xem[j] = Xtemp ;
# endfor
wa = np.zeros(L+1)
t = np.zeros(L+1); t[0] = 0.0; wa[0] = Xzero
for j in range(1,L+1):
    t[j] = Dt + j*Dt
    wa[j] = Xem[j-1]
# endfor
##  plot([0:Dt:T],[Wzero,WA],'r--*'), hold off
plt.plot(t, wa, 'r-')
# plot([0:Dt:T],[Xzero,Xem],'r--*'), hold off
plt.xlabel('t')
plt.ylabel('W')
plt.title('Euler-Maruyama-Verfahren')
plt.legend(["Exakte Loesung", "Euler-Maruyama-Loesung"], loc ="upper right")
plt.show()
emerr = abs(wa[L-1] - Xtrue[N-1])
# emerr = abs(Xem(end)-Xtrue(end))
# Programmende
