#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math

# Programm 9.1 Loesung der Poisson-Gleichung (pdenr_gb.m)
# auf einen Nichtrechteckgebietmit Dirichlet- und Neumann-RB
# input:  
# output: Loesung u(x,y) auf dem Gitter x,y
# Aufruf: [u,x,y,it] = pdenr_gb

def pdenr_gb():
# Materialdaten, Quellen
    la = 1.0; q = 1.0; ff = 0.0
# Gebeitslaenge, Gebietshoehe
    ll = 1; hh = 10.5*ll/16.0; ll1 = ll/2.0; hh1 = hh*3.0/7.0
    mm = (hh-hh1)/(ll1-ll); nn = hh-mm*ll1
# Dimensionierung des Gitters
    nf = 6 # nf = 3
    n = nf*8+1; m = nf*7+1
# Gitterinitialisierung
    x = np.zeros(n); y = np.zeros(m)
    x[0] = 0.0; y[0] = 0.0
    h=1/(nf*8); k=3*h/4
    for i in range(1,n):
        x[i] = i*h
    for j in range(1,m):
        y[j] = j*k
# Initialisierung der Gebietsmatrix
    mg = [[0 for j in range(m)] for i in range(n)]
    mg = np.dot(1,mg)
    for i in range(n):
        for j in range(m):
            mg[i,j] = 0
#       endfor
#   endfor
    for i in range(1,n-1):
        for j in range(1,m-1):
            mg[i,j] = 1
#       endfor
#   endfor
    for i in range(1,n-1):
        for j in range(1,m-1):
            if (j == m-2):
                mg[i,j] = 3
            if (i == 0):
                mg[i,j] = 5
            if (j == 0):
                mg[i,j] = 7
            if (i == n-2):
                mg[i,j] = 9
#       endfor
#   endfor
    mg[0,m-1] = 4; mg[0,0] = 6; mg[n-1,0] = 8
    for i in range(0,n-1):
        for j in range(0,m-1):
            if (abs(mm*x[i]+nn-y[j]) < 1.0e-06):
                mg[i,j] = 2
            if (y[j] > mm*x[i]+nn):
                mg[i,j] = 0
#       endfor
#   endfor
# Startfeld incl. homogener Dirichlet-Randbedingungen
    u = [[0 for j in range(m)] for i in range(n)]
    u = np.dot(1.0,u)
    ualt = [[0 for j in range(m)] for i in range(n)]
    ualt = np.dot(1.0,ualt)
# Konstanten
    omega = 1.65; h2=h*h; k2=k*k 
    dd1=1/(2/h2+2/k2); dd2=1/(1/h2+1/k2); dd3=1/(2/h2+3/k2); dd4=1/(3/h2+3/k2)
    dd5=1/(3/h2+2/k2)
    dd6=dd4; dd7=dd3; dd8=dd4; dd9=dd5
    whk=q*math.sqrt(h2+k2)/(h*k)/la
    udiff = 1; unorm = 1; it = 0
    la2=2*la;
# Randwerte
    un=0; us=0; uo=0; uw=0
# sor-Iteration
    while (math.sqrt(udiff/unorm) > 1.0e-05 and it < 1200):
        for i in range(n-1):
            for j in range(m-1):
                ualt[i,j] = u[i,j]
        for i in range(n-1):
            for j in range(m-1):
                if (mg[i,j] == 1):
                    u[i,j] = dd1*(ff/la + (u[i+1,j] + u[i-1,j])/h2 + (u[i,j+1]+u[i,j-1])/k2)
                if (mg[i,j] == 2):
                    u[i,j] = dd2*(ff/la2 + u[i-1,j]/h2 + u[i,j-1]/k2 + whk)
                if (mg[i,j] == 3):
                    u[i,j] = dd3*(ff/la + (u[i+1,j] + u[i-1,j])/h2 + (2*un+u[i,j-1])/k2)
                if (mg[i,j] == 4):
                    u[i,j] = dd4*(ff/la + (u[i+1,j] + 2*uw)/h2 + (2*un+u[i,j-1])/k2)
                if (mg[i,j] == 5):
                    u[i,j] = dd5*(ff/la + (u[i+1,j] + 2*uw)/h2 + (u[i,j+1]+u[i,j-1])/k2)
                if (mg[i,j] == 6):
                    u[i,j] = dd6*(ff/la + (u[i+1,j] + 2*uw)/h2 + (u[i,j+1]+2*us)/k2)
                if (mg[i,j] == 7):
                    u[i,j] = dd7*(ff/la + (u[i+1,j] + u[i-1,j])/h2 + (u[i,j+1]+2*us)/k2)
                if (mg[i,j] == 8):
                    u[i,j] = dd8*(ff/la + (2*uo + u[i-1,j])/h2 + (u[i,j+1]+2*us)/k2)
                if (mg[i,j] == 9):
                    u[i,j] = dd9*(ff/la + (2*uo + u[i-1,j])/h2 + (u[i,j+1]+u[i,j-1])/k2)
#           endfor
#       endfor
        udiff = 0.0; unorm = 0.0
        for i in range(n-1):
            for j in range(m-1):
                u[i,j] = omega*u[i,j]+(1-omega)*ualt[i,j]
                udiff = udiff + math.pow(u[i,j]-ualt[i,j],2)
                unorm = unorm + math.pow(ualt[i,j],2)
#           endfor
#       endfor
        if (unorm == 0.0):
            unorm = 1
#       endif
        it = it + 1
#   endwhile
    for i in range(n):
        for j in range(m):
            if (mg[i,j] == 0):
                u[i,j] = 0.0
#       endfor
#   endfor                
    return (u,x,y,it)
# endfunction

def main():
    u, x, y, it = pdenr_gb()

    import numpy as np
    import matplotlib.pyplot as plt
    from mpl_toolkits.mplot3d import Axes3D

    X, Y = np.meshgrid(x, y, indexing='ij')
    fig = plt.figure()
    axes = plt.axes(projection='3d')

    Z = u
    plt.xlabel('x')
    plt.ylabel('y')

    axes.set_title('Surface plot of pde solution')
    axes.plot_surface(X, Y, Z)

    plt.show()


if __name__ == '__main__':
    main()

