#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math

# Programm 3.1 zur QR-Zerlegung einer Matrix c0 = q*r
# Erzeugung der wesentlichen Informationen fuer 
# die orthogonale Matrix Q und der oberen Dreiecksmatrix R
# input:  Matrix c=(c(ij)) vom Typ m x p gegeben (m >= p)
# output: Matrix c mit den wesentlichen Informationen
#         der orthogonalen Matrix q und der oberen Dreiecksmatrix r,
#         Erzeugung des inputs fuer das Programm bestapprqr_gb.m,
# Aufruf: [c,t] = qr_gb(c);

def qr_gb(c0):
    m = len(c0)
    p = len(c0[0])
    c = np.zeros((m,p))
    t = np.zeros(p)
    for i in range(m):
        for j in range(p):
            c[i,j] = c0[i,j]
    for l in range(0,p): # mslice[1:p]:
        gamma = 0
        for i in range(l,m): # mslice[l:m]:
            gamma = gamma + c[i,l]*c[i,l]
#       endfor
        gamma = math.sqrt(gamma)
        if (c[l,l] < 0):
            gamma = -gamma
#       endif
        peta = math.sqrt(2*gamma*(gamma + c[l,l]))
        t[l] = -gamma #        t(l).lvalue = -gamma
        c[l,l] = (c[l,l] + gamma)/peta #        c(l, l).lvalue = (c(l, l) + gamma) / peta
        for k in range(l+1,m): # mslice[l + 1:m]:
            c[k,l] = c[k,l]/peta # c(k, l).lvalue = c(k, l) / peta
#       endfor
        for j in range(l+1,p): # mslice[l + 1:p]:
            g = 0
            for k in range(l,m): # mslice[l:m]:
                g = g + c[k,l]*c[k,j]
#           endfor
            g = g + g
            for i in range(l,m): # mslice[l:m]:
                c[i,j] = c[i,j] - g*c[i,l] # c(i, j).lvalue = c(i, j) - g * c(i, l)
#           endfor
#       endfor
#   endfor
    return (c, t)
#   endfunction

# Programm 3.2 zur Bestimmung der Bestapproximation x
# zur Behandlung des moeglicherweise ueberbestimmten Systems C x = y
# input:  Matrix C vom Typ m x p gegeben (m >= p), und rechte Seite y
# output: Loesung x, Residuenvektor d
# Aufruf: [x,d] = quadapprox_gb(c,y);

def quadapprox_gb(c0,y0):

    c, t = qr_gb(c0)

    m = len(c) # length(c(mslice[:], 1))
    p = len(c[0]) # length(c(1, mslice[:]))
    x = np.zeros(p) # list(range(0,p))
    d = np.zeros(m) # list(range(0,m))
    y = np.zeros(m)
    for i in range(m):
        y[i] = y0[i]
# Aufbau der rechten Seite Q^T y
    for l in range(0,p): # mslice[1:p]:
        summe = 0
        for k in range(l,m): # mslice[l:m]:
            summe = summe + c[k,l]*y[k]
#       endfor 
        summe = summe + summe
        for k in range(l,m): # mslice[l:m]:
            y[k] = y[k] - summe*c[k,l] # y(k).lvalue = y(k) - summe * c(k, l)
#       endfor
#   endfor
#   Loesung des Gleichungssystems R x = Q^T y
    for l in range(0,p): # mslice[p:-1:1]:
        i = p-1-l
        summe = y[i]
        d[i] = 0 # d(i).lvalue = 0
        if ((i + 1) <= p):
            for k in range(i+1,p): # mslice[i + 1:p]:
                summe = summe - c[i,k]*x[k]
#           endfor
#       endif
        x[i] = summe/t[i] # x(i).lvalue = summe / t(i)     
#   endfor
    if ((p + 1) <= m):
        for i in range(p,m): # mslice[p + 1:m]:
            d[i] = y[i] # d(i).lvalue = y(i)
#       endfor
#   endif      
#   Ruecktransformation des Residuums
    for i in range(0,p): # mslice[p:-1:1]:
        l = p-i-1
        summe = 0
        for k in range(l,m): # mslice[l:m]:
            summe = summe + c[k,l]*d[k]
#       endfor  
        summe = summe + summe
        for k in range(l,m): #  mslice[l:m]:
            d[k] = d[k] - summe*c[k,l] # d(k).lvalue = d(k) - summe * c(k, l)
#       endfor
#   endfor
#    x
#    d
    return (x,d)
#   endfunction

def main():
    a = np.array([[2, -1 ,0],[-1, 2, -1],[0,-1,2]])
    b = np.array([1,0,1])

    x, d = quadapprox_gb(a,b)

    print(x)

if __name__ == '__main__':
    main()

