#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math
import matplotlib.pyplot as plt

# Programm zur Bereitstellung 
# der rechten Differentialgleichungsseite f(x,y)
def f(x,y):
    r = y/(1.0 + x) + (x+1)*math.cos(x)
    return r
# endfunction

# Programm 8.1  (ruku4_gb.py)
# zur Loesung von gew. DGL 1. Ordnung y'=f(x,y), y(x0)=y0
# klassisches Runge-Kutta-Verfahren 4. Ordnung
# input:  Schrittweite h, Intervallgrenzen x0,x1, Anfangswert y0=y(x0) 
# output: Loesung y(x) an den Punkten x
# benutztes Programm: Nutzerfunktion f (rechte Seite der Dgl.)
# Aufruf: [y,x] = ruku4_gb(x0,x1,y0,h);

def ruku4_gb(x0,x1,y0,h):
    n = int((x1 - x0)/h)
    x = np.zeros(n+1); y = np.zeros(n+1)
    yn = y0; y[0] = y0
    xn = x0; x[0] = x0
    for k in range(0,n):
        k1 = f(xn,yn)
        k2 = f(xn + 0.5*h,yn + 0.5*h*k1)
        k3 = f(xn + 0.5*h,yn + 0.5*h*k2)
        k4 = f(xn + h,yn + h*k3)
        yn = yn + h*(k1 + 2.0*k2 + 2.0*k3 + k4)/6; xn = xn + h
        x[k+1] = xn; y[k+1] = yn
#   endfor
    return (y,x)
# endfunction

def main():
    x0 = 0
    x1 = 1
    y0 = 0
    h = 0.005

    y,x = ruku4_gb(x0,x1,y0,h)

    nn = len(x)
    yy = np.ones(nn)
    for i in range(nn):
        yy[i] = (1 + x[i])*math.sin(x[i])

    plt.plot(x, y, color="blue", linewidth=1,linestyle="-")
    plt.plot(x, yy, color="red", linewidth=1,linestyle="--")
    plt.show()

    print(x,y)
    print(nn)

if __name__ == '__main__':
    main()

