#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import matplotlib.pyplot as plt

# Programm 8.4a
def laf(x):
    y = x
    return y
# endfunction

# Programm 8.4b
def kf(x):
    y = -4*x
    return y
# endfunction

# Programm 8.4c
def cf(x):
    y = 0
    return y
# endfunction

# Programm 2.4 zur Loesung 
# eines tridiagonalen linearen Gleichungssystems
#   -pc(k)*yps(k-1)+pd(k)*yps(k)-pe(k)*yps(k+1)=pf(k),k=1,n
# mit einer schwach diagonal dominanten Koeffizientenmatrix!
# input:
#   n - Endindex 
#   pc, pd, pe - Vektoren der Dimension l der tridiag. Matrix
#   pf         - Vektor der rechten Seiten
#   pc(1) und pe(n) sind dummy-Elemente
# output
#   yps        - Ergebnisvektor
# c und d sind Hilfsfelder
# aufruf: yps = tridia_gb(pc,pd,pe,pf,n);

def tridia_gb(pc,pd,pe,pf,n):
# function [yps]= tridia_gb(pc,pd,pe,pf,n);
    c = list(range(0,n))
    d = list(range(0,n))
    yps = list(range(0,n))
    c[0] = pe[0]/pd[0];
    for i in range(1,n-1): #  i=2:n-1
        c[i] = pe[i]/(pd[i] - c[i-1]*pc[i])
#   endfor
    d[0] = pf[0]/pd[0]
    for i in range(1,n): # i=2:n
        d[i] = (pf[i] + d[i-1]*pc[i])/(pd[i] - c[i-1]*pc[i]);
#   endfor
    yps[n-1] = d[n-1]
    for k in range(0,n-1): # i=n-1:-1:1
        i = n-k-2
        yps[i] = d[i] + c[i]*yps[i+1]
#   endfor
    return yps        
# endfunction


# Programm 8.4 zur Loesung eines Zweipunkt-Randwertproblmes (rwp2p2_gb.py)
# -(laf(x)y')' + kf(x)y = cf(x),\; y'(a)=eta0, y(b)=eta1
# vorzugeben: a, b, eta0, eta1, n
# input:  Intervallgrenzen a,b, Randwerte eta0, eta1, n=(b-a)/h
# output: Loesung yy(x) an den Stellen x
# Benutzung der Programme tridia_gb (Programm 2.4), laf, kf, cf
# Aufruf: [yy,x] = rwp2p2_gb(a,b,eta0,eta1,n);

def rwp2p2_gb(a,b,eta0,eta1,n):
    h = (b-a)/n
    x = np.zeros(n+1); xp = np.zeros(n+1)
    dx = np.zeros(n+1); dxp = np.zeros(n+1)
    dd = np.zeros(n-1); cc = np.zeros(n-1); ee = np.zeros(n-1)
    bb = np.zeros(n-1)
    y = np.zeros(n-1); yy = np.zeros(n+1)
    x[0]=a
    for k in range(n):
        x[k+1] = x[0] + (k+1)*h
    for k in range(n):
        xp[k] = x[k] + 0.5*h
    for k in range(n-1):
        dx[k] = xp[k+1]-xp[k]
    for k in range(n):
        dxp[k] = x[k+1]-x[k]
    dd[0] = laf(xp[1])/(dxp[1]*dx[0])+kf(x[1])
    cc[0] = 0
    for k in range(1,n-1):
        dd[k] = laf(xp[k+1])/(dxp[k+1]*dx[k])+laf(xp[k])/(dxp[k]*dx[k])+kf(x[k+1])
        cc[k] = laf(xp[k])/(dxp[k]*dx[k])
    for k in range(0,n-2):
        ee[k] = laf(xp[k+1])/(dxp[k+1]*dx[k])
    ee[n-2] = 0
    bb[0] = cf(x[1]) + eta1*laf(xp[0])/dx[0]
    for k in range(1,n-2):
        bb[k] = cf(x[k+1])
    bb[n-2] = cf(x[n-1])+eta1*laf(xp[n-1])/(dxp[n-1]*dx[n-2]);
# Loesung
    y = tridia_gb(cc,dd,ee,bb,n-1)
    yy[0] = y[0] - dxp[0]*eta0
    for k in range(1,n):
        yy[k] = y[k-1]
    yy[n] = eta1
    return (yy,x)
# endfunction

def main():
    a = 0
    b = 8
    eta0 = 0
    eta1 = -0.1749
    n = 200

    yy,x = rwp2p2_gb(a,b,eta0,eta1,n)

    print(x,yy)

    plt.plot(x, yy, color="blue", linewidth=1,linestyle="-")
    plt.show()

if __name__ == '__main__':
    main()

