#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math

# Programm 8.5a
# klassisches Runge-Kutta-Verfahren 4. Ordnung
def rungek4fs(x,y01,y02,h,n):
    yn1 = y01; yn2 = y02
    y1 = np.zeros(n+1); y2 = np.zeros(n+1)
    y1[0] = y01; y2[0] = y02; xn = x[0]
    for k in range(0,n):
        k11 = fs1(xn,yn1,yn2)
        k12 = fs2(xn,yn1,yn2)
        k21 = fs1(xn+0.5*h,yn1+0.5*h*k11,yn2+0.5*h*k12)
        k22 = fs2(xn+0.5*h,yn1+0.5*h*k11,yn2+0.5*h*k12)
        k31 = fs1(xn+0.5*h,yn1+0.5*h*k21,yn2+0.5*h*k22)
        k32 = fs2(xn+0.5*h,yn1+0.5*h*k21,yn2+0.5*h*k22)
        k41 = fs1(xn+h,yn1+h*k31,yn2+h*k32)
        k42 = fs2(xn+h,yn1+h*k31,yn2+h*k32)
        yn1 = yn1 + h*(k11+2.0*k21+2.0*k31+k41)/6.0
        yn2 = yn2 + h*(k12+2.0*k22+2.0*k32+k42)/6.0
        xn = x[k+1]; y1[k+1] = yn1; y2[k+1] = yn2
#   endfor
    return (y1,y2)
# endfunction

# Programm 8.5b
# klassisches Runge-Kutta-Verfahren 4. Ordnung
def rungek4fsy(x,y,y01,y02,h,n):
    yn1 = y01
    yn2 = y02
    y1 = np.zeros(n+1)
    y2 = np.zeros(n+1)
    y1[0] = y01
    y2[0] = y02
    xn = x[0]
    for k in range(0,n):
        yk = y[k]
        k11 = fs1(xn,yn1,yn2)
        k12 = fsy2(xn,yk,yn1,yn2)
        k21 = fs1(xn+0.5*h,yn1+0.5*h*k11,yn2+0.5*h*k12)
        k22 = fsy2(xn+0.5*h,yk,yn1+0.5*h*k11,yn2+0.5*h*k12)
        k31 = fs1(xn+0.5*h,yn1+0.5*h*k21,yn2+0.5*h*k22)
        k32 = fsy2(xn+0.5*h,yk,yn1+0.5*h*k21,yn2+0.5*h*k22)
        k41 = fs1(xn+h,yn1+h*k31,yn2+h*k32)
        k42 = fsy2(xn+h,yk,yn1+h*k31,yn2+h*k32)
        yn1 = yn1 + h*(k11+2.0*k21+2.0*k31+k41)/6.0
        yn2 = yn2 + h*(k12+2.0*k22+2.0*k32+k42)/6.0
        xn = x[k+1]
        y1[k+1] = yn1
        y2[k+1] = yn2
#   endfor
    return (y1,y2)
# endfunction

# Programm 8.5c
def fs1(x,y1,y2):
    y = y2
    return y
# endfunction

# Programm 8.5d
def fs2(x,y1,y2):
    y = x/(1+y1*y1)
    return y
# endfunction

# Programm 8.5e
def fsy2(x,yk,y1,y2):
    y = -y1*2*x*yk/math.pow((1+yk*yk),2)
    return y
# endfunction


# Programm 8.5 Schiessverfahren zur Loesung eines 2-Punkt-RWP (schiessv_gb.m)
#  y''=fs(x,y), y(a)=eta0, y(b)=eta1
# input:  Intervallgrenzen a,b, n Anzahl der Teilintervalle, eta0,eta1
#         zeta als Startwert des Newton-Verfahrens fuer y'(a)
# output: Loesung y1(x) an den Stellen x
# benutzte Programme: fs1,fs2,fsy2 fuer die rechten Seiten der AWP's
#         rungek4fs(x,y01,y11,h,n),rungek4fsy(x,y1,y2,y02,y12,h,n)
#         zur Implementierung des klassischen RK-Verfahrens 4.Ordn.
# Aufruf: [y1,x] = schiessv_gb(a,b,eta0,eta1,n,zeta);

def schiessv_gb(a,b,eta0,eta1,n,zeta):
    h = (b-a)/n
    x = np.zeros(n+1); y1 = np.zeros(n+1); y2 = np.zeros(n+1)
    yy1 = np.zeros(n+1); yy2 = np.zeros(n+1)
    for k in range(0,n+1):
        x[k] = a + k*h
#   endfor
    it = 0; g = 1; y02 = 0; y12 = 1; y01 = eta0
    while (abs(g) > 0.000001 and it < 10):
        y11 = zeta
        [y1,y2] = rungek4fs(x,y01,y11,h,n)
        g = y1[n] - eta1
#
        [yy1,yy2] = rungek4fsy(x,y1,y02,y12,h,n)
        gy = yy1[n]
        zeta = zeta - g/gy
        print('zeta=',zeta)
        print('g=',g)
        it = it + 1
#   endwhile
    return (y1,x,it)
# endfunction

def main():
    a = 0
    b = 2
    eta0 = 0
    eta1 = 1
    n = 100
    zeta = 0

    y1,x,it = schiessv_gb(a,b,eta0,eta1,n,zeta)

    print(y1,x)
    print(it)

if __name__ == '__main__':
    main()

