#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np

# Programm 2.4 zur Loesung 
# eines tridiagonalen linearen Gleichungssystems
#   -pc(k)*yps(k-1)+pd(k)*yps(k)-pe(k)*yps(k+1)=pf(k),k=1,n
# mit einer schwach diagonal dominanten Koeffizientenmatrix!
# input:
#   n - Endindex 
#   pc, pd, pe - Vektoren der Dimension l der tridiag. Matrix
#   pf         - Vektor der rechten Seiten
#   pc(1) und pe(n) sind dummy-Elemente
# output
#   yps        - Ergebnisvektor
# c und d sind Hilfsfelder
# aufruf: yps = tridia_gb(pc,pd,pe,pf,n);

def tridia_gb(pc,pd,pe,pf,n):
# function [yps]= tridia_gb(pc,pd,pe,pf,n);
    c = list(range(0,n))
    d = list(range(0,n))
    yps = list(range(0,n))
    c[0] = pe[0]/pd[0];
    for i in range(1,n-1): #  i=2:n-1
        c[i] = pe[i]/(pd[i] - c[i-1]*pc[i])
#   endfor
    d[0] = pf[0]/pd[0]
    for i in range(1,n): # i=2:n
        d[i] = (pf[i] + d[i-1]*pc[i])/(pd[i] - c[i-1]*pc[i]);
#   endfor
    yps[n-1] = d[n-1]
    for k in range(0,n-1): # i=n-1:-1:1
        i = n-k-2
        yps[i] = d[i] + c[i]*yps[i+1]
#   endfor
    return yps        
# endfunction


# Programm 5.5a zur Bestimmung der 2. Ableitungen
# function [ys] = splabl(x,y,n);
# Interpolation mit echten Splines/Bestimmung der Ableitungen ***
# Benutzung von tridia (Programm 2.4): -c xi-1 + d xi -e xi+1 = b ***

def splabl(x,y,n):
    ys = list(range(0,n))
    c = list(range(0,n)); d = list(range(0,n))
    e = list(range(0,n)); b = list(range(0,n))
#    
    for i in range(0,n-1): # i=1:n-1
        ys[i]=x[i+1]-x[i]
#   endfor
    for i in range(1,n-1): # i=2:n-1
        c[i-1]=-ys[i-1]
        d[i-1]=2*(ys[i-1]+ys[i])
        e[i-1]=-ys[i]
#   endfor
    c[0]=0.0 
    e[n-2]=0.0
    for i in range(1,n-1): # i=2:n-1
        b[i-1]=6*((y[i+1]-y[i])/ys[i]-(y[i]-y[i-1])/ys[i-1])
#   endfor
# *** Bestimmung der Ableitungen ys(1)...ys(n)
    xs = tridia_gb(c,d,e,b,n-2)
    ys[0]=0
    ys[n-1]=0
    for i in range(1,n-1): # i=2:n-1
        ys[i]=xs[i-1]
#   endfor
    return ys
# endfunction

# Programm 5.5b zur Berechnung 
# eines Funktionswertes eines kubischen Splines
# function [splinew] 
# wobei x(1) <= z <= x(n) vorausgesetzt wird

def splber(z,x,y,ys,n):
    i = 0 # i=1
    while (x[i] > z or z > x[i+1]):
        i = i+1
#   endwhile
    iz=i
    h=x[iz+1]-x[iz]
    z1=z-x[iz]
    a0=y[iz]
    a1=(y[iz+1]-y[iz])/h - h*(2*ys[iz]+ys[iz+1])/6
    a2=ys[iz]/2
    a3=(ys[iz+1]-ys[iz])/(6*h)
    splinew = a0+(a1+(a2+a3*z1)*z1)*z1
    return splinew
# endfunction


# Programm 5.5 zur Berechnung eines kubischen Splines
# Berechnung des Wertes s(xx) eines kubischen Splines
# fuer die Stuetzpunkte (x_0,y_0),...,(x_n,y_n)
# input:  Vektoren x(1:n), y(1:n), Stelle xx
# output: Wert des Splines s(xx)
# berechnet wird ein interpolierender Spline
# benutzte Funktionen: splabl, splber
# aufruf: fwert = spline_gb(xx,x,y);
def spline_gb(xx,x,y):
    n = len(x) # n=length(x);
    ys = splabl(x,y,n)
    fwert = splber(xx,x,y,ys,n)
    return fwert
# endfunction

def main():
    x = np.array([1,2,3,4,5])
    y = np.array([0,2,3,2,1])
    xx = 3.5

    fwert = spline_gb(xx,x,y)

    print(fwert)

if __name__ == '__main__':
    main()

