"""
Programm 10.1: strong_wiener_gb.py , starke Approximation von W

Wiener-Prozess über [0,1] mit dt = 2^(-10))
starke Approximation auf Gitter mit Dt = R*dt
"""

import numpy as np
import matplotlib.pyplot as plt


def main():
    np.random.seed(50)  # constant seeding for random package

    t0, T = 0.0, 1.0
    Wzero = 0.0

    N = 2**10
    dt = (T - t0)/N
    dW = (dt**0.5)*np.random.randn(N)  # Inkremente
    W = np.cumsum(dW)  # Wiener-Prozess
    Xtrue = np.array(W)
    plt.plot([0.0 + j*dt for j in range(N + 1)], np.hstack(([Wzero], W)), 'g-', label='Wiener-Prozess')

    R = 4
    L = int(N/R)  # L EM steps of size Dt = R*dt
    Dt = R*dt
    WA = np.zeros(shape=(L,))  # Allokierung
    for j in range(L):
        Winc = sum(dW[R*j:R*(j + 1)])
        WA[j] = WA[max(0, j - 1)] + Winc
    plt.plot([0.0 + j*Dt for j in range(L + 1)], np.hstack(([Wzero], WA)), 'r--*', label='Starke Approxiamtion')

    plt.xlabel('t')
    plt.ylabel('W')
    plt.title('Approx. eines Wiener-Prozesses')
    plt.legend()

    print(f'emerr = {abs(WA[-1] - Xtrue[-1])}')

    plt.show()


if __name__ == '__main__':
    main()
